@@ -1,24 +1,28 @@
 """
 Serve the elasticsearch API as a threaded Flask app.
+
+TODO: restoring persistence requires re-starting of cluster instances.
 """
 import json
 import logging
 import threading
 import time
 from random import randint
-from typing import Dict, Optional
+from typing import Dict
 
+from botocore.utils import ArnParser
 from flask import Flask, jsonify, make_response, request
 
-from localstack import config, constants
-from localstack.constants import ELASTICSEARCH_URLS, TEST_AWS_ACCOUNT_ID
+from localstack.constants import ELASTICSEARCH_DEFAULT_VERSION, TEST_AWS_ACCOUNT_ID
 from localstack.services import generic_proxy
-from localstack.services.es.cluster import ProxiedElasticsearchCluster
+from localstack.services.es import versions
+from localstack.services.es.cluster_manager import ClusterManager, create_cluster_manager
 from localstack.services.generic_proxy import RegionBackend
 from localstack.utils import persistence
 from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import get_service_protocol, poll_condition, to_str
+from localstack.utils.common import synchronized, to_str
+from localstack.utils.serving import Server
 from localstack.utils.tagging import TaggingService
 
 LOG = logging.getLogger(__name__)
@@ -26,8 +30,6 @@ LOG = logging.getLogger(__name__)
 APP_NAME = "es_api"
 API_PREFIX = "/2015-01-01"
 
-DEFAULT_ES_VERSION = "7.10"
-
 DEFAULT_ES_CLUSTER_CONFIG = {
     "InstanceType": "m3.medium.elasticsearch",
     "InstanceCount": 1,
@@ -40,101 +42,84 @@ DEFAULT_ES_CLUSTER_CONFIG = {
 # timeout in seconds when giving up on waiting for the cluster to start
 CLUSTER_STARTUP_TIMEOUT = 600
 
-# ideally, each domain gets its own cluster. to save resources, we currently re-use the same
-# cluster instance. this also means we lie to the client about the the elasticsearch domain
-# version. the first call to create_domain with a specific version will create the cluster
-# with that version. subsequent calls will believe they created a cluster with the version
-# they specified.
-_cluster: Optional[ProxiedElasticsearchCluster] = None
-
-# mutex for modifying domains
-_domain_mutex = threading.Lock()
-
+# flask app serving the API endpoints
 app = Flask(APP_NAME)
 app.url_map.strict_slashes = False
 
+# mutex for modifying domains
+_domain_mutex = threading.RLock()
+
+# cluster manager singleton
+_cluster_manager = None
+
+
+@synchronized(_domain_mutex)
+def cluster_manager() -> ClusterManager:
+    global _cluster_manager
+    if not _cluster_manager:
+        _cluster_manager = create_cluster_manager()
+    return _cluster_manager
+
 
 class ElasticsearchServiceBackend(RegionBackend):
     # maps cluster names to cluster details
-    es_clusters: Dict[str, ProxiedElasticsearchCluster]
+    es_clusters: Dict[str, Server]
     # storage for domain resources (access should be protected with the _domain_mutex)
     es_domains: Dict[str, Dict]
     # static tagging service instance
     TAGS = TaggingService()
 
     def __init__(self):
-        self.es_clusters = {}
-        self.es_domains = {}
+        self.es_clusters = dict()
+        self.es_domains = dict()
 
 
-def _run_cluster_startup_monitor(cluster):
-    region = ElasticsearchServiceBackend.get()
+def _run_cluster_startup_monitor(cluster: Server, domain_name: str, region: str):
     LOG.debug("running cluster startup monitor for cluster %s", cluster)
+
     # wait until the cluster is started, or the timeout is reached
-    status = poll_condition(cluster.is_up, timeout=CLUSTER_STARTUP_TIMEOUT, interval=5)
-
-    LOG.debug("cluster state polling returned! status = %s", status)
+    is_up = cluster.wait_is_up(CLUSTER_STARTUP_TIMEOUT)
 
+    LOG.debug("cluster state polling for %s returned! status = %s", domain_name, is_up)
     with _domain_mutex:
-        LOG.debug("iterating over cluster domains %s", region.es_clusters.keys())
-        for domain, domain_cluster in region.es_clusters.items():
-            LOG.debug("checking cluster for domain %s", domain)
-            if cluster is domain_cluster:
-                if domain in region.es_domains:
-                    region.es_domains[domain]["Created"] = status
+        status = ElasticsearchServiceBackend.get(region).es_domains[domain_name]
+        status["Processing"] = False
 
 
-def _create_cluster(domain_name, data):
+def _create_cluster(domain_name: str, data: Dict):
     """
-    Create a new entry in ES_DOMAINS if the domain does not yet exist. Start a ElasticsearchCluster if this is the first
-    domain being created. NOT thread safe, needs to be called around _domain_mutex.
+    Uses the ClusterManager to create a new cluster for the given domain_name in the region of the current request
+    context. NOT thread safe, needs to be called around _domain_mutex.
     """
-    global _cluster
     region = ElasticsearchServiceBackend.get()
+    arn = get_domain_arn(domain_name)
 
-    if _cluster:
-        # see comment on _cluster
-        LOG.info("elasticsearch cluster already created, using existing one for %s", domain_name)
-        region.es_clusters[domain_name] = _cluster
-        data["Created"] = _cluster.is_up()
-        return
+    manager = cluster_manager()
+    cluster = manager.create(arn, data)
 
-    # creating cluster for the first time
-    version = data.get("ElasticsearchVersion") or DEFAULT_ES_VERSION
-    _cluster = ProxiedElasticsearchCluster(
-        port=config.PORT_ELASTICSEARCH, host=constants.LOCALHOST, version=version
-    )
-    LOG.info("starting %s on %s:%s", type(_cluster), _cluster.host, _cluster.port)
-    _cluster.start()
-    region.es_clusters[domain_name] = _cluster
+    region.es_clusters[domain_name] = cluster
 
-    # run a background thread that will update all domains that use this cluster to set
-    # data['Created'] = <status> once it is started, or the CLUSTER_STARTUP_TIMEOUT is reached
-    # FIXME: if the cluster doesn't start, these threads will stay open until the timeout is
-    #  reached, even if the cluster is already shut down. we could fix this with an additional
-    #  event, or a timer instead of Poll, but it seems like a rare case in the first place.
-    threading.Thread(target=_run_cluster_startup_monitor, daemon=True, args=(_cluster,)).start()
+    # FIXME: in AWS, the Endpoint is set once the cluster is running, not before (like here), but our tests and
+    #  in particular cloudformation currently relies on the assumption that it is set when the domain is created.
+    data["Endpoint"] = cluster.url.split("://")[-1]
+
+    if cluster.is_up():
+        data["Processing"] = False
+    else:
+        # run a background thread that will update all domains that use this cluster to set
+        # the cluster state once it is started, or the CLUSTER_STARTUP_TIMEOUT is reached
+        threading.Thread(
+            target=_run_cluster_startup_monitor,
+            args=(cluster, domain_name, region.name),
+            daemon=True,
+        ).start()
 
 
-def _cleanup_cluster(domain_name):
-    global _cluster
+def _remove_cluster(domain_name: str):
     region = ElasticsearchServiceBackend.get()
-    cluster = region.es_clusters.pop(domain_name)
-
-    LOG.debug(
-        "cleanup cluster for domain %s, %d domains remaining", domain_name, len(region.es_clusters)
-    )
-
-    if not region.es_clusters:
-        # because cluster is currently always mapped to _cluster, we only shut it down if no other
-        # domains are using it
-        LOG.info("shutting down elasticsearch cluster after domain %s cleanup", domain_name)
-        cluster.shutdown()
-        # FIXME: if delete_domain() is called, then immediately after, create_domain() (without
-        #  letting time pass for the proxy to shut down) there's a chance that there will be a bind
-        #  exception when trying to start the proxy again (which is currently always bound to
-        #  PORT_ELASTICSEARCH)
-        _cluster = None
+    arn = get_domain_arn(domain_name)
+    cluster_manager().remove(arn)
+    del region.es_clusters[domain_name]
 
 
 def error_response(error_type, code=400, message="Unknown error."):
@@ -148,6 +133,16 @@ def error_response(error_type, code=400, message="Unknown error."):
     return response, code
 
 
+def get_domain_arn(domain_name: str, region: str = None, account_id: str = None) -> str:
+    return aws_stack.elasticsearch_domain_arn(
+        domain_name=domain_name, account_id=account_id, region_name=region
+    )
+
+
+def parse_domain_arn(arn: str):
+    return ArnParser().parse_arn(arn)
+
+
 def get_domain_config_status():
     return {
         "CreationDate": "%.2f" % time.time(),
@@ -173,7 +168,7 @@ def get_domain_config(domain_name):
                     aws_stack.get_region(),
                     TEST_AWS_ACCOUNT_ID,
                     domain_name,
-                ),  # noqa: E501
+                ),
                 "Status": config_status,
             },
             "AdvancedOptions": {
@@ -221,13 +216,15 @@ def get_domain_config(domain_name):
             "LogPublishingOptions": {
                 "Options": {
                     "INDEX_SLOW_LOGS": {
-                        "CloudWatchLogsLogGroupArn": "arn:aws:logs:%s:%s:log-group:sample-domain"
-                        % (aws_stack.get_region(), TEST_AWS_ACCOUNT_ID),  # noqa: E501
+                        "CloudWatchLogsLogGroupArn": "arn:aws:logs:%s:%s:log-group:%s"
+                        % (aws_stack.get_region(), TEST_AWS_ACCOUNT_ID, domain_name),
+                        # noqa: E501
                         "Enabled": False,
                     },
                     "SEARCH_SLOW_LOGS": {
-                        "CloudWatchLogsLogGroupArn": "arn:aws:logs:%s:%s:log-group:sample-domain"
-                        % (aws_stack.get_region(), TEST_AWS_ACCOUNT_ID),  # noqa: E501
+                        "CloudWatchLogsLogGroupArn": "arn:aws:logs:%s:%s:log-group:%s"
+                        % (aws_stack.get_region(), TEST_AWS_ACCOUNT_ID, domain_name),
+                        # noqa: E501
                         "Enabled": False,
                     },
                 },
@@ -246,6 +243,10 @@ def get_domain_config(domain_name):
                 },
                 "Status": config_status,
             },
+            "DomainEndpointOptions": {
+                "Options": status.get("DomainEndpointOptions", {}),
+                "Status": config_status,
+            },
         }
     }
 
@@ -255,17 +256,13 @@ def get_domain_status(domain_name, deleted=False):
     status = region.es_domains.get(domain_name) or {}
     cluster_cfg = status.get("ElasticsearchClusterConfig") or {}
     default_cfg = DEFAULT_ES_CLUSTER_CONFIG
-    endpoint = "%s://%s:%s" % (
-        get_service_protocol(),
-        config.HOSTNAME_EXTERNAL,
-        config.PORT_ELASTICSEARCH,
-    )
-    return {
+
+    result = {
         "DomainStatus": {
-            "ARN": "arn:aws:es:%s:%s:domain/%s"
-            % (aws_stack.get_region(), TEST_AWS_ACCOUNT_ID, domain_name),
-            "Created": status.get("Created", False),
+            "ARN": get_domain_arn(domain_name, region.name, TEST_AWS_ACCOUNT_ID),
+            "Created": True,
             "Deleted": deleted,
+            "Processing": status.get("Processing", True),
             "DomainId": "%s/%s" % (TEST_AWS_ACCOUNT_ID, domain_name),
             "DomainName": domain_name,
             "ElasticsearchClusterConfig": {
@@ -284,9 +281,8 @@ def get_domain_status(domain_name, deleted=False):
                     "ZoneAwarenessEnabled", default_cfg["ZoneAwarenessEnabled"]
                 ),
             },
-            "ElasticsearchVersion": status.get("ElasticsearchVersion") or DEFAULT_ES_VERSION,
-            "Endpoint": endpoint,
-            "Processing": False,
+            "ElasticsearchVersion": status.get("ElasticsearchVersion")
+            or ELASTICSEARCH_DEFAULT_VERSION,
             "EBSOptions": {
                 "EBSEnabled": True,
                 "VolumeType": "gp2",
@@ -297,6 +293,11 @@ def get_domain_status(domain_name, deleted=False):
         }
     }
 
+    if status.get("Endpoint"):
+        result["DomainStatus"]["Endpoint"] = status.get("Endpoint")
+
+    return result
+
 
 @app.route("%s/domain" % API_PREFIX, methods=["GET"])
 def list_domain_names():
@@ -314,12 +315,15 @@ def create_domain():
     with _domain_mutex:
         if domain_name in region.es_domains:
             # domain already created
-            return error_response(error_type="ResourceAlreadyExistsException")
+            return error_response(
+                error_type="ResourceAlreadyExistsException",
+                message=f"domain {domain_name} already exists in region {region.name}",
+            )
 
         # "create" domain data
         region.es_domains[domain_name] = data
 
-        # lazy-init the cluster, and set the data["Created"] flag
+        # lazy-init the cluster (sets the Endpoint and Processing flag of the domain status)
         _create_cluster(domain_name, data)
 
         # create result document
@@ -381,7 +385,7 @@ def delete_domain(domain_name):
 
         result = get_domain_status(domain_name, deleted=True)
         del region.es_domains[domain_name]
-        _cleanup_cluster(domain_name)
+        _remove_cluster(domain_name)
 
     # record event
     event_publisher.fire_event(
@@ -396,7 +400,7 @@ def delete_domain(domain_name):
 @app.route("%s/es/versions" % API_PREFIX, methods=["GET"])
 def list_es_versions():
     result = []
-    for key in ELASTICSEARCH_URLS.keys():
+    for key in versions.install_versions.keys():
         result.append(key)
     return jsonify({"ElasticsearchVersions": result})
 
@@ -404,10 +408,23 @@ def list_es_versions():
 @app.route("%s/es/compatibleVersions" % API_PREFIX, methods=["GET"])
 def get_compatible_versions():
     result = [
-        {"SourceVersion": "6.5", "TargetVersions": ["6.7", "6.8"]},
+        {"SourceVersion": "7.10", "TargetVersions": []},
+        {"SourceVersion": "7.9", "TargetVersions": ["7.10"]},
+        {"SourceVersion": "7.8", "TargetVersions": ["7.9", "7.10"]},
+        {"SourceVersion": "7.7", "TargetVersions": ["7.8", "7.9", "7.10"]},
+        {"SourceVersion": "7.4", "TargetVersions": ["7.7", "7.8", "7.9", "7.10"]},
+        {"SourceVersion": "7.1", "TargetVersions": ["7.4", "7.7", "7.8", "7.9", "7.10"]},
+        {"SourceVersion": "6.8", "TargetVersions": ["7.1", "7.4", "7.7", "7.8", "7.9", "7.10"]},
         {"SourceVersion": "6.7", "TargetVersions": ["6.8"]},
-        {"SourceVersion": "6.8", "TargetVersions": ["7.1"]},
-        {"SourceVersion": "7.1", "TargetVersions": ["7.4", "7.7"]},
+        {"SourceVersion": "6.5", "TargetVersions": ["6.7", "6.8"]},
+        {"SourceVersion": "6.4", "TargetVersions": ["6.5", "6.7", "6.8"]},
+        {"SourceVersion": "6.3", "TargetVersions": ["6.4", "6.5", "6.7", "6.8"]},
+        {"SourceVersion": "6.2", "TargetVersions": ["6.3", "6.4", "6.5", "6.7", "6.8"]},
+        {"SourceVersion": "6.0", "TargetVersions": ["6.3", "6.4", "6.5", "6.7", "6.8"]},
+        {"SourceVersion": "5.6", "TargetVersions": ["6.3", "6.4", "6.5", "6.7", "6.8"]},
+        {"SourceVersion": "5.5", "TargetVersions": ["5.6"]},
+        {"SourceVersion": "5.3", "TargetVersions": ["5.6"]},
+        {"SourceVersion": "5.1", "TargetVersions": ["5.6"]},
     ]
     return jsonify({"CompatibleElasticsearchVersions": result})
 
